<?php

namespace Drupal\cforge\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\migrate\MigrateSkipRowException;

/**
 * Migration modifications
 */
class MigrationSubscriber implements EventSubscriberInterface {

  const FILTER_FORMATS = [
    'editor_full_html' => 'full_html',
    'editor_filtered_html' => 'basic',
    'filtered_html' => 'basic',
    'full_html' => 'full_html',
    'plain_text' => ''
  ];

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
       //AKA Drupal\migrate\Event\MigrateEvents::PRE_IMPORT
       'migrate.pre_row_save' => [['migratePreRowSave']]
    ];
  }

  /**
   * @param Drupal\migrate\Event\MigratePreRowSave $event
   */
  public function migratePreRowSave($event) {
    $row = $event->getRow();
    $migration = $event->getMigration();
    if ($migration->id() == 'd7_field' or $migration->id() == 'd7_field_instance') {
      $field_name = $row->getSourceProperty('field_name');
      //Skip these d7 fields and rename them when importing the entities they belong to.
      $existing_fields = [
        'body',
        'cforge_docs_categories',
        'event_date',
        'galleries',
        'image',
        'manage_notes',
        'offers_wants_categories',
        'offers_wants_types',
        'profile_notes',
        'profile_phones',
        'profile_address',
        'upload', //became 'attached'
        'upload_private', // Became 'attached',
        //'title', // Prevents 'base_field_override' fields being created
      ];
      if (in_array($field_name, $existing_fields)) {
        throw new MigrateSkipRowException('Not migrating hamlets '. $migration->id() .' '. $field_name);
      }
    }

    // Field instances showing on entity form displays
    if ($migration->id() == 'd7_field_instance_widget_settings') {
      if ($row->getSourceProperty('field_name') == 'profile_address') {
        // The addressfield available_countries and default country are stored in
        // the d7 field config instance widget setting.
        // Needs to go to the user.user.address field setting
        $settings = $row->getSourceProperty('widget')['settings'];
        \Drupal\field\Entity\FieldConfig::load('user.user.address')
          ->setSetting('available_countries', $settings['available_countries'])
          ->setSetting('default_country', $settings['default_country'])
          ->save();
        throw new MigrateSkipRowException("Migrated widget settings only from user address widget");
      }
      else {
        // Ensure any fields which have been added by user 1 are visible on the profile tab.
        // Don't bother about the weight
        $row->setDestinationProperty('form_mode', 'profile');
      }
    }


    if($migration->id() == 'd7_user') {
      // Rename the fields
      $map = [
        'manage_notes' => 'notes_admin',
        'profile_notes' => 'notes',
        'profile_phones' => 'phones',
        'profile_address' => 'address',
      ];
      foreach ($map as $old_name => $new_name) {
        if ($row->hasDestinationProperty($old_name)) {
          $row->setDestinationProperty($new_name, $row->getDestinationProperty($old_name));
          $row->removeDestinationProperty($old_name);
        }
      }
    }

    // Move all terms that used to be in offers_wants_categories field.
    if($migration->id() == 'd7_node' or $migration->id() == 'd7_mcapi_transaction') {
      if ($row->hasDestinationProperty('offers_wants_categories')) {
        $entity_type = $row->getSourceProperty('entity_type');
        $new_field_name = $row->getSourceProperty('entity_type') == 'node' ? 'categories' : 'category';
        $row->setDestinationProperty($new_field_name, $row->getSourceProperty('offers_wants_categories'));
        $row->removeDestinationProperty('offers_wants_categories');
      }
    }
    // Change the filter format for every body field
    // I don't know why this doesn't happen already
    if($row->getSourceProperty('plugin') == 'd7_node') { //what about d7_comment?
      $this->entityBodyFilterFormat($row, 'body');

      // Change the name of the image taxonomy field
      if ($migration->id() == 'd7_node:image') {
        $row->setDestinationProperty('gallery', $row->getDestinationProperty('galleries'));
        $row->removeDestinationProperty('galleries');
      }
      // Change the name of the document taxonomy field
      if ($migration->id() == 'd7_node:document') {
        $row->setDestinationProperty('doctype', $row->getDestinationProperty('cforge_docs_categories'));
        $row->removeDestinationProperty('cforge_docs_categories');
      }
    }

    if ($migration->id() == 'd7_taxonomy_vocabulary') {
      if (in_array($row->getSourceProperty('machine_name'), ['galleries', 'cforge_docs_categories'])) {
        throw new MigrateSkipRowException();
      }
    }
    // Move the terms to the new gallery name
    if ($migration->id() == 'd7_taxonomy_term:galleries') {
      $row->setDestinationProperty('vid', 'galleries');
    }
    elseif ($migration->id() == 'd7_taxonomy_term:cforge_docs_categories') {
      $row->setDestinationProperty('vid', 'binders');
    }

    if($migration->id() == 'd7_comment' and $row->getDestinationProperty('entity_type') != 'smallad') {
      $row->setDestinationProperty('comment_type', $row->getDestinationProperty('entity_type'));
      $row->setDestinationProperty('field_name', 'comments');
      $row->setDestinationProperty('body', $row->getDestinationProperty('comment_body'));// Assumes no comments on transactions
    }

    if ($migration->id() == 'contact_category') {
       //echo "\nChecking contact import:"; print_r($row->getDestination());
    }

    if ($migration->id() == 'd7_action') {
       $callback = $row->getSourceProperty('callback');
       if (substr($callback, 0, 22) == 'views_bulk_operations_') {
         if (!\Drupal::moduleHandler()->moduleExists('views_bulk_operations')) {
           throw new MigrateSkipRowException('Views Bulk Operations module not installed.');
         }
       }
       elseif (!is_callable($callback)) {
         throw new MigrateSkipRowException('Callback not found: '.$callback);
       }
    }

  }

//    givenName: givenName
//    familyName: familyName
//    administrativeArea: administrativeArea
//    locality: locality
//    dependentLocality: dependentLocality
//    postalCode: postalCode
//    addressLine1: addressLine1
//    addressLine2: addressLine2
//    sortingCode: '0'
//    organization: '0'
//    recipient: '0'


  /**
   *
   * @param Row $row
   * @param string $body_field_name
   *
   * @deprecated
   */
  private function entityBodyFilterFormat($row, $body_field_name) {
    if ($bods = $row->getDestinationProperty($body_field_name)) {
      $map = static::FILTER_FORMATS;
      foreach ($bods as $delta => &$body) {
        if ($new_format = $map[$body['format']]) {
          $bods[$delta]['format'] = $new_format;
        }
      }
      $row->setDestinationProperty($body_field_name, $bods);
    }
  }

}
